@@ -66,7 +66,6 @@ codeOffsetPairSize = struct.calcsize(codeOffsetPairFormat)
 
 
 class table_E_B_L_C_(DefaultTable.DefaultTable):
-
     dependencies = ["EBDT"]
 
     # This method can be overridden in subclasses to support new formats
@@ -76,7 +75,6 @@ class table_E_B_L_C_(DefaultTable.DefaultTable):
         return eblc_sub_table_classes[indexFormat]
 
     def decompile(self, data, ttFont):
-
         # Save the original data because offsets are from the start of the table.
         origData = data
         i = 0
@@ -138,7 +136,6 @@ class table_E_B_L_C_(DefaultTable.DefaultTable):
                 curStrike.indexSubTables.append(indexSubTable)
 
     def compile(self, ttFont):
-
         dataList = []
         self.numSizes = len(self.strikes)
         dataList.append(sstruct.pack(eblcHeaderFormat, self))
@@ -297,14 +294,13 @@ class Strike(object):
 
 
 class BitmapSizeTable(object):
-
     # Returns all the simple metric names that bitmap size table
     # cares about in terms of XML creation.
     def _getXMLMetricNames(self):
         dataNames = sstruct.getformat(bitmapSizeTableFormatPart1)[1]
-        dataNames = dataNames + sstruct.getformat(bitmapSizeTableFormatPart2)[1]
+        dataNames = {**dataNames, **sstruct.getformat(bitmapSizeTableFormatPart2)[1]}
         # Skip the first 3 data names because they are byte offsets and counts.
-        return dataNames[3:]
+        return list(dataNames.keys())[3:]
 
     def toXML(self, writer, ttFont):
         writer.begintag("bitmapSizeTable")
@@ -476,14 +472,12 @@ class EblcIndexSubTable(object):
 # are very similar. The only difference between them is the size per offset
 # value. Code put in here should handle both cases generally.
 def _createOffsetArrayIndexSubTableMixin(formatStringForDataType):
-
     # Prep the data size for the offset array data format.
     dataFormat = ">" + formatStringForDataType
     offsetDataSize = struct.calcsize(dataFormat)
 
     class OffsetArrayIndexSubTableMixin(object):
         def decompile(self):
-
             numGlyphs = self.lastGlyphIndex - self.firstGlyphIndex + 1
             indexingOffsets = [
                 glyphIndex * offsetDataSize for glyphIndex in range(numGlyphs + 2)
@@ -625,7 +619,6 @@ class eblc_index_sub_table_3(
 
 class eblc_index_sub_table_4(EblcIndexSubTable):
     def decompile(self):
-
         (numGlyphs,) = struct.unpack(">L", self.data[:4])
         data = self.data[4:]
         indexingOffsets = [
